local L = LibStub("AceLocale-3.0"):GetLocale("QuestItem")
--~ local media = LibStub:GetLibrary("LibSharedMedia-3.0");

QuestItem.BagBackdrop = {
	bgFile = "Interface\\AddOns\\QuestItem\\Textures\\BagFrameBackground",
	edgeFile = "Interface\\AddOns\\QuestItem\\Textures\\BagFrameBorder",
	tile = true,
	edgeSize = 32,
	tileSize = 32,
	insets = {left = 5, right = 5, top = 5, bottom = 5}
}

----------------------
-- Frame Routines
----------------------

function QuestItem:CreateBagFrame()
	if not self.BagFrame then
		self.BagFrame = CreateFrame("Frame", "QuestItemBag", UIParent)
	end
	if not self.bags then
		self.bags = {}
		for i=-1,4 do
			self.bags[i] = CreateFrame("Frame", "QuestItemBag"..i, self.BagFrame)
			self.bags[i]:SetID(i)
		end
	end
	self.BagFrame:SetToplevel(true)
	self.BagFrame:EnableMouse(true)
	self.BagFrame:SetMovable(true)
	self.BagFrame:SetFrameStrata("MEDIUM")
	self.BagFrame:SetFrameLevel(2)
	self.BagFrame:SetWidth(185)
	self.BagFrame:RegisterForDrag("LeftButton")
	self.BagFrame:SetPoint(self.db.char.point, self.db.char.relativeTo, self.db.char.relativePoint, self.db.char.xOffset, self.db.char.yOffset)
	self.BagFrame:SetScript("OnDragStart", 
		function()
			self.BagFrame:StartMoving();
		end );
	self.BagFrame:SetScript("OnDragStop",
		function()
			self.BagFrame:StopMovingOrSizing();
			self.BagFrame:SetScript("OnUpdate", nil);
			self:FinishedMoving(self.db.char, self.BagFrame);
		end );
	self.BagFrame:SetScript("OnHide", function() self:QuestBag_OnHide() end)
	
	self.BagFrame.textureContainer = CreateFrame("Frame", nil, self.BagFrame)
	self.BagFrame.textureContainer:SetBackdrop(self.BagBackdrop)
	self.BagFrame.textureContainer:SetBackdropColor(1,1,1,1)
	self.BagFrame.textureContainer:SetPoint("TOPLEFT", self.BagFrame, "TOPLEFT")
	self.BagFrame.textureContainer:SetPoint("BOTTOMRIGHT", self.BagFrame, "BOTTOMRIGHT")
	self.BagFrame.textureContainer:SetAlpha(1)
	self.BagFrame.textureContainer:Show()
	self.BagFrame.portrait = self.BagFrame.textureContainer:CreateTexture(nil, "ARTWORK")
	self.BagFrame.portrait:SetWidth(34)
	self.BagFrame.portrait:SetHeight(34)
	self.BagFrame.portrait:SetPoint("TOPLEFT", self.BagFrame, "TOPLEFT", 3, -5)
	self.BagFrame.portrait:SetTexture("Interface\\Icons\\INV_Box_01")
	self.BagFrame.portrait:Show()
	local portraitFrame = self.BagFrame.textureContainer:CreateTexture(nil, "OVERLAY")
 	portraitFrame:SetTexture("Interface\\AddOns\\QuestItem\\Textures\\BagFramePortraitFrame")
	portraitFrame:SetTexCoord(0, 0.745, 0, 0.745)
	portraitFrame:SetWidth(48)
	portraitFrame:SetHeight(48)
	portraitFrame:SetPoint("TOPLEFT", self.BagFrame, "TOPLEFT", -4, 0)
	portraitFrame:Show()
	self.BagFrame.name = self.BagFrame.textureContainer:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
	self.BagFrame.name:SetWidth(112)
	self.BagFrame.name:SetHeight(12)
	self.BagFrame.name:SetPoint("TOPRIGHT", self.BagFrame, "TOPRIGHT", -33, -9)
	self.BagFrame.name:SetText("Quest Item Bag")
	local closeButton = CreateFrame("Button", nil, self.BagFrame.textureContainer, "UIPanelCloseButton")
	closeButton:SetScript("OnClick", function() self:QuestBag_OnHide() end)
	closeButton:SetPoint("TOPRIGHT", self.BagFrame, "TOPRIGHT", 0, -1)
end

function QuestItem:UpdateBagFrame()
	-- check all items in bags and add to quest bag frame 
	self:ClearQuestBag()
	for bag = 0,4 do
		for slot = 1, GetContainerNumSlots(bag) do
			local itemLink = GetContainerItemLink(bag, slot)
			if itemLink then -- check if its a quest item
				local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(itemLink)
				if itemType == L["Quest"] then
					self:AddItemtoQuestBag(bag, slot, itemLink)
				end
			end
		end
	end
	self:RedrawBagFrame()
end

function QuestItem:RedrawBagFrame()
	local rows = self.db.char.questbag.rows
	self.BagFrame:SetHeight((rows + 1)*40 + 16)
	for i = 1, self.db.char.questbag.count do
		local questitem = self.db.char.questbag[i]
		local itemButton = self:SetupItemButton(i, questitem.bag, self.db.char.questbag.count)
		local texture, itemCount, locked, quality, readable = GetContainerItemInfo(questitem.bag, questitem.slot)
		SetItemButtonCount(itemButton, itemCount)
		SetItemButtonDesaturated(itemButton, locked, 0.5, 0.5, 0.5)
		itemButton.readable = readable
		if texture then
			SetItemButtonTexture(itemButton, texture)
		else
			SetItemButtonTexture(itemButton, "Interface\\AddOns\\QuestItem\\Textures\\EmptySlot")
		end
		SetItemButtonNormalTextureVertexColor(itemButton, 1, 1, 1)
		itemButton.questitem = questitem
		itemButton:SetParent(self.bags[questitem.bag])
		itemButton:SetID(questitem.slot)
		itemButton:Show()
	end
	for i = self.db.char.questbag.count + 1, rows * 4 do
 		local itemButton = self:SetupItemButton(i, -1, rows * 4)
		SetItemButtonCount(itemButton, 0)
		SetItemButtonDesaturated(itemButton, false, 0.5, 0.5, 0.5)
		SetItemButtonTexture(itemButton, "Interface\\AddOns\\QuestItem\\Textures\\EmptySlot")
		SetItemButtonNormalTextureVertexColor(itemButton, 1, 1, 1)
		itemButton:SetParent(self.bags[-1])
		itemButton:SetID(0)
		itemButton:Show()
	end
end

function QuestItem:SetupItemButton(index, bag, max)
	local frameName = self.BagFrame:GetName()
	local buttonName = frameName.."Item"..index
	local itemButton = getglobal(buttonName)
	if not itemButton then
		itemButton = CreateFrame("Button", buttonName, self.bags[bag], "ContainerFrameItemButtonTemplate")
		self:HookScript(itemButton, "OnEnter", "ItemButton_OnEnter")
	end
	itemButton:ClearAllPoints()
	if (index == 1) then
		-- put the buttom bottom right
		itemButton:SetPoint("BOTTOMRIGHT", self.BagFrame, "BOTTOMRIGHT", -13, 13)
	elseif (mod((index-1), 4) == 0) then
		itemButton:SetPoint("BOTTOMRIGHT", frameName.."Item"..(index - 4), "TOPRIGHT", 0, 3)
	else
		itemButton:SetPoint("BOTTOMRIGHT", frameName.."Item"..(index - 1), "BOTTOMLEFT", -3, 0)
	end
	return itemButton
end

function QuestItem:ItemButton_OnEnter(button, ...)
	if button then
		if (button:GetRight() >= (GetScreenWidth()/2)) then
			GameTooltip:SetOwner(button, "ANCHOR_LEFT")
		else
			GameTooltip:SetOwner(button, "ANCHOR_RIGHT")
		end
		if self.db.char.debug and button.questitem then
			self:Print("button bag:"..button.questitem.bag.." slot:"..button.questitem.slot.." parent bag :"..button:GetParent():GetID().." identified slot:"..button:GetID())
		else
			self:Print("parent bag :"..button:GetParent():GetID().." identified slot:"..button:GetID())
		end
	end
end
